#include <ddraw.h>
#include <dinput.h>
#include <sys/timeb.h>
#include <stdio.h>
#include <math.h>
#include "Direct.h"

HRESULT ReadBMP(char *csFile, LPDIRECTDRAWSURFACE7 Surface) 
{
	HDC hdcImage, hdc = NULL; 
	BITMAP bm;
	DDSURFACEDESC2 ddsd;
	HBITMAP hbm;
	DWORD dwRet;
	hbm = (HBITMAP)LoadImage(NULL, csFile, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
	if(!hbm) {
		dwRet = GetLastError();
		return dwRet;
	}

	GetObject(hbm, sizeof(BITMAP), &bm);
	Surface->Restore();

	hdcImage = CreateCompatibleDC(NULL);

	SelectObject(hdcImage, hbm);
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
	Surface->GetSurfaceDesc(&ddsd);

	dwRet = Surface->GetDC(&hdc);
	if (dwRet != DD_OK) {
		return dwRet;
	}

	dwRet = StretchBlt(hdc, 0, 0, 
				ddsd.dwWidth, 
				ddsd.dwHeight, 
				hdcImage, 0, 0, 
				bm.bmWidth, 
				bm.bmHeight, 
				SRCCOPY);
	if(dwRet == 0) {
		dwRet = GetLastError();
	}
	else {
		dwRet = ERROR_SUCCESS;
	} 

	Surface->ReleaseDC(hdc);

	DeleteObject(hbm);
	DeleteDC(hdcImage);
	return dwRet;
}

/***************************************************************************/
#define RES_X 800
#define RES_Y 600
#define RES_B 32
#define CLASS_N "Direct Draw Test"
#define WIN_N "Simulace"

LPDIRECTDRAWSURFACE7	OffSurf;

struct	_timeb start, stop;
int		i = 0;
double	fps = 0;
char	sfps[20];
HDC		hdc;

void FPS() {
	if (m_lpDDSBack->GetDC(&hdc) == DD_OK) {
		sprintf(sfps, "FPS: %.1f", fps);
		SetBkColor( hdc, RGB( 0, 0, 0 ) );
		SetTextColor( hdc, RGB( 0, 0, 255 ) );
		TextOut( hdc, 0, 0, sfps, lstrlen(sfps) );
		m_lpDDSBack->ReleaseDC(hdc);
	}
	i ++;
	if (i > 50)
	{
		_ftime(&stop);
		fps = i/((stop.time - start.time) + 0.001*(stop.millitm - start.millitm));
		_ftime(&start);
		i = 0;
	}
}

DDCOLORKEY d = {RGB(255,255,255),RGB(255,255,255)};
LPDIRECTDRAWSURFACE7 OffRot;

void rotuj(LPDIRECTDRAWSURFACE7 src, LPDIRECTDRAWSURFACE7 des, double u, int x, int y) {
	HDC hdc_src, hdc_des;
	double su = sin(u), cu = cos(u);

	DDSURFACEDESC2 ddsd_src, ddsd_des;

	ddsd_src.dwSize = sizeof(ddsd_src);
	ddsd_src.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
	src->GetSurfaceDesc(&ddsd_src);

	ddsd_des.dwSize = sizeof(ddsd_des);
	ddsd_des.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
	des->GetSurfaceDesc(&ddsd_des);

	DDBLTFX dblx;
	RECT rcdes = {0, 0, ddsd_des.dwWidth, ddsd_des.dwHeight};
	ZeroMemory(&dblx, sizeof(dblx));
	dblx.dwSize = sizeof(dblx);
	dblx.dwFillColor = RGB(255, 255, 255);
	des->Blt(&rcdes, NULL, NULL, DDBLT_WAIT|DDBLT_COLORFILL , &dblx);

	src->Lock(NULL, &ddsd_src, DDLOCK_WAIT, NULL);
	des->Lock(NULL, &ddsd_des, DDLOCK_WAIT, NULL);
	unsigned char *pix_src, *pix_des;
	pix_src = (unsigned char *)(ddsd_src.lpSurface);
	pix_des = (unsigned char *)(ddsd_des.lpSurface);
/*
	int i, j;
	int xx, yy;
	int d1 = ddsd_des.dwWidth*0.5, d2 = ddsd_des.dwHeight*0.5;
	int d3, d4, d5, d6, d7;
	for (i = 0; i < ddsd_src.dwWidth; i ++)
		for(j = 0; j < ddsd_src.dwHeight; j ++) {
			d5 = 4*i + j*ddsd_src.lPitch; 
//			if ((pix_src[d5] < 251)&&(pix_src[d5+1] < 251)&&(pix_src[d5+1] < 251))
			{
				xx = ((i - x)*su + (j - y)*cu + d1);
				yy = ((i - x)*cu - (j - y)*su + d2);
				d3 = 4*xx + yy*ddsd_des.lPitch;
				pix_des[d3] = pix_src[d5];
				pix_des[d3+1] = pix_src[d5+1];
				pix_des[d3+2] = pix_src[d5+2];
				pix_des[d3+3] = 0;
			}
		}
*/
	int i, j;
	int xx, yy;
	int d1 = ddsd_des.dwWidth*0.5, d2 = ddsd_des.dwHeight*0.5;
	int d3, d4, d5;
	for (i = 0; i < ddsd_des.dwWidth; i ++)
		for(j = 0; j < ddsd_des.dwHeight; j ++) {
			xx = ((i - d1)*su + (j - d2)*cu + x);
			yy = ((i - d1)*cu - (j - d2)*su + y);
			if ((xx >= 0)&&(xx < ddsd_src.dwWidth)&&(yy >= 0)&&(yy < ddsd_src.dwHeight))
			{
				d5 = 4*i + j*ddsd_des.lPitch;
				d3 = 4*xx + yy*ddsd_src.lPitch;
				pix_des[d5] = pix_src[d3];
				pix_des[d5+1] = pix_src[d3+1];
				pix_des[d5+2] = pix_src[d3+2];
				pix_des[d5+3] = 0;
			}
		}

	src->Unlock(NULL); 
	des->Unlock(NULL); 
}

RECT obd = {0, 0, 75, 75};
RECT offobd = {0, 0, 60, 45};
RECT rotobd = {0, 0, 75, 75};
DDBLTFX dblx;
RECT rcdes = {0, 0, RES_X, RES_Y};

void InitOther() {
	_ftime(&start);
	i = 0;
	fps = 0;
	CreateSurface(offobd.right, offobd.bottom, &OffSurf);
	ReadBMP("Auto03.bmp", OffSurf);
	CreateSurface(rotobd.right, rotobd.bottom, &OffRot);
	OffRot->SetColorKey(DDCKEY_SRCBLT, &d);
	
	ZeroMemory(&dblx, sizeof(dblx));
	dblx.dwSize = sizeof(dblx);
	dblx.dwFillColor = RGB(0, 0, 0);
}

void CleanOther() {
	if ( m_lpDD ) {
		if( OffSurf )	OffSurf->Release();
		if( OffRot )	OffRot->Release();		
		m_lpDDSFront	= NULL;
		m_lpDDSBack		= NULL;
	}
}

double a = 4, v = 0.1, aa = 0;
double px = 100, py = 100, vx = 0, vy = 0;

char     buffer[256]; 
void Drive() {
 
    lpdiKeyboard->GetDeviceState(sizeof(buffer),(LPVOID)&buffer); 
    if (buffer[DIK_UP]&0x80) v += 0.08; 
    if (buffer[DIK_DOWN]&0x80) v -= 0.08; 
    if (buffer[DIK_RIGHT]&0x80) a -= v*0.008; 
    if (buffer[DIK_LEFT]&0x80) a += v*0.008;
	v *= 0.99;
}

void move() {
	vy = -cos(a)*v;
	vx = -sin(a)*v;
	px += vx;
	py += vy;

	obd.left = px - rotobd.right*0.5;
	obd.top = py - rotobd.bottom*0.5;
	obd.right = obd.left + rotobd.right;
	obd.bottom = obd.top + rotobd.bottom;

	if (obd.left < 0) {
		obd.left = 0;
		vx *= -1;
		px = rotobd.right*0.5;
	}
	if (obd.right >= RES_X) {
		obd.right = RES_X;
		vx *= -1;
		px = RES_X - rotobd.right*0.5;
	}
	if (obd.top < 0) {
		obd.top = 0;
		vy *= -1;
		py = rotobd.bottom*0.5;
	}
	if (obd.bottom >= RES_Y) {
		obd.bottom = 0;
		vy *= -1;
		py = RES_Y - rotobd.bottom*0.5;
	}
}

void Render() {
	move();
	m_lpDDSBack->Blt(&rcdes, NULL, NULL, DDBLT_WAIT|DDBLT_COLORFILL , &dblx);

	if (a != aa) rotuj(OffSurf, OffRot, a, offobd.right*0.5, offobd.bottom*0.5);
	aa = a;
	m_lpDDSBack->Blt(&obd, OffRot, &rotobd, DDBLT_WAIT|DDBLT_KEYSRC , NULL); 

	FPS();

	m_lpDDSFront->Flip( NULL, DDFLIP_WAIT );
}


INT WINAPI WinMain( HINSTANCE hInst, HINSTANCE, LPSTR, INT ) {
	WNDCLASSEX	wc;
	HWND		hWnd;
	BOOL		gm = TRUE;
	MSG			msg;

	InitWin( &wc, &hWnd, CLASS_N, WIN_N, 100, 100, 200, 200 );

    if( SUCCEEDED( InitDDraw( hWnd, RES_X, RES_Y, RES_B ) ) ) {
		InitOther();
        ShowWindow( hWnd, SW_SHOWDEFAULT );
        UpdateWindow( hWnd );
		InitDInput( hWnd );

		while ( gm ) {
			if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) ) {
				if( gm = GetMessage( &msg, NULL, 0, 0 ) ) {
					if (msg.message == WM_KEYDOWN) {
						if (msg.wParam == 27) gm = false;
					}
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
			}
			else {
				Render();
				Drive();
			}
		}
    }

	CleanDInput();
    CleanOther();
    CleanDDraw();
    CleanWin(&wc, CLASS_N);

    return 0;
}
